package controllers_pages

import (
	"main/mvc/models"
	"main/mvc/models/models_pages"
	"net/http"
	"strings"
)

// Обработчик главной страницы
func PostPageHandler(app *models.App) http.HandlerFunc {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		postLink := r.URL.Path

		// Ссылки на посты имеют вид postLink = /link/, а если прилетело что то типо /link/123123,
		// то надо оставить только часть /link/
		secondSlash := strings.IndexByte(postLink[1:], '/')
		if secondSlash != -1 {
			postLink = postLink[:secondSlash+2]
		}

		// Страничка рендерится только если ее нет в кэше
		page, ok := app.PagesCache.Get(postLink)
		if !ok {
			post := app.Posts[models_pages.PostLink(postLink)]
			pageData, err := models_pages.RenderPostPage(app.Templates, app.Version, post.Data)
			if err != nil {
				http.Error(w, err.Error(), http.StatusInternalServerError)
				return
			}
			ETag := models.GenerateETag(pageData)
			page = &models.Page{Data: pageData, ETag: ETag}
			app.PagesCache.Set(postLink, page)
		}

		sendPostPage(r, w, page)
	})
}

// Отправляет страницу
func sendPostPage(r *http.Request, w http.ResponseWriter, page *models.Page) {
	w.Header().Set("Cache-Control", "public, max-age=31536000")
	w.Header().Set("ETag", page.ETag)
	w.Header().Set("Content-Type", "text/html; charset=utf-8")

	if match := r.Header.Get("If-None-Match"); match == page.ETag {
		w.WriteHeader(http.StatusNotModified)
		return
	}

	w.Write(page.Data)
}
