package controllers_pages

import (
	"main/mvc/models"
	"main/mvc/models/models_pages"
	"net/http"
)

// Обработчик главной страницы
func PostsPageHandler(app *models.App) http.HandlerFunc {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		// Страничка рендерится только если ее нет в кэше
		page, ok := app.PagesCache.Get(models_pages.PostsPageTmplName)
		if !ok {
			pageData, err := app.Posts.RenderPostsPage(app.Templates, app.Version)
			if err != nil {
				http.Error(w, err.Error(), http.StatusInternalServerError)
				return
			}
			ETag := models.GenerateETag(pageData)
			page = &models.Page{Data: pageData, ETag: ETag}
			app.PagesCache.Set(models_pages.PostsPageTmplName, page)
		}

		sendPostsPage(r, w, page)
	})
}

// Отправляет страницу
func sendPostsPage(r *http.Request, w http.ResponseWriter, page *models.Page) {
	w.Header().Set("Cache-Control", "public, max-age=31536000")
	w.Header().Set("ETag", page.ETag)
	w.Header().Set("Content-Type", "text/html; charset=utf-8")

	if match := r.Header.Get("If-None-Match"); match == page.ETag {
		w.WriteHeader(http.StatusNotModified)
		return
	}

	w.Write(page.Data)
}
